var scheduler;
var appointmentsChanged = false;
const defaultAppointmentContext = "self"

$(document).ready(async function() {

    if ($("#scheduler").length > 0) {
        // The following are used in include_appointment_info and ViewAppointments
        if ($("#RequestLink").val() == "") {
            $("#AppointmentID").empty();
            $("#NewAppointment").prop("disabled", true)
        }
        $("#RequestLink").change(function() { GetAppointments(); });

        $("#Site").change(async function(){ GetReadingRooms(); });
        if ($("#Site option[value!='']").length < 2) { 
            $(".sites-enabled").addClass("d-none"); 
            GetReadingRooms();
        }

        $("#ReadingRoomID").change(function () {
            window.readingRoom = readingRooms.filter(obj => { return obj.ID === $("#ReadingRoomID").val() })[0];
            RefreshScheduler();
        });

        $('#SchedulerModal').on('shown.bs.modal', function () {
            GetReadingRooms().then(() => {
                RefreshScheduler();
            });
        })

        $('#SchedulerModal').on('hidden.bs.modal', function () {
            if (appointmentsChanged == true){
                window.location.reload();
            }
        });

        GetReadingRooms();

    } else {

        // The following are used on the EditAppointment form
        $('#ScheduledDate').change(async function() {
            let startTimes = await GetValidStartTimes($('#ScheduledDate').val())
            $('#StartTime').data("persisted-value", $('#StartTime').val())
            $('#StartTime').empty();
            if (startTimes.length == 0) {
                let option = document.createElement("option");
                option.setAttribute("value", null);
                option.innerText = "No appointments available"
                $('#StartTime').append(option);

                $('#StopTime').empty();
                let endOption = document.createElement("option");
                endOption.setAttribute("value", null);
                endOption.innerText = "No appointments available"
                $('#StopTime').append(endOption);
            } else {
                startTimes.forEach( endTime => {
                    let option = document.createElement("option");
                    option.setAttribute("value", endTime.format("YYYY-MM-DDTHH:mm:ss.sss"));
                    option.innerText = endTime.format("LT");
                    $('#StartTime').append(option);
                });
                SelectPersistedValue($('#StartTime'));
                $('#StartTime').change();    
            }
        });

        $('#StartTime').change(function () {
            let endTimes = GetValidEndTimes($('#StartTime').val(), GetAvailabilityForTime(new moment($('#StartTime').val())));
            $('#StopTime').data("persisted-value", $('#StopTime').val())
            $('#StopTime').empty();
            if (endTimes.length == 0) {
                let option = document.createElement("option");
                option.setAttribute("value", null);
                option.innerText = "No appointments available"
                $('#StopTime').append(option);
            } else {
                endTimes.forEach( endTime => {
                    let option = document.createElement("option");
                    option.setAttribute("value", endTime.format("YYYY-MM-DDTHH:mm:ss.sss"));
                    option.innerText = endTime.format("LT");
                    $('#StopTime').append(option);
                });
                SelectPersistedValue($('#StopTime'));
            }
        });

        await new Promise((resolve, reject) => {
            $.ajax({
                type: "GET",
                url: "aeon.dll/ajax?query=GetReadingRooms",
                data: {},
                success: function (response) { 
                    if (response.status == "success") {
                        window.readingRoom = response.data.filter(obj => { return obj.ID === $("#ReadingRoomID").val() })[0];
                        resolve(response.data);
                    } else {
                        HandleError(response);
                        reject(response.message);
                    }
                },
                error: function () {
                    HandleError("Server error. Please refresh the page and try again. If this error persists, please contact staff.");
                    reject("Server Error");
                }
            });
        });

        $('#ScheduledDate').val( new Date($('#ScheduledDate').val()).toLocaleDateString() );
        $('#ScheduledDate').datepicker();
        $('#ScheduledDate').change();


        SelectPersistedValue($('#StartTime'));
        SelectPersistedValue($('#StopTime'));
    }

});

function HandleError(errorResponse) {
    if (errorResponse.status == "fail") {
        ShowError(errorResponse.message);
    } else {
        ShowError("A server error occurred. Please refresh the page and try again. If this error persists, please contact staff.");
    }
}

function ShowError(message) {
    $("#SchedulerError").text(message);
    $("#ModalDismiss").click();
}

async function RefreshScheduler() {
    GetAvailability(new moment().add(readingRoom.MinLeadDays, 'd'))
        .then(() => { return GetAppointments(); })
        .then(() => { BuildSchedulerControl(); },
    () => {
        ShowError("Unable to retrieve availability information. If this error persists, please contact staff.")
    });
}

async function GetReadingRooms() {
    // Cache the current value, so we don't lose it when reloading the list
    if ($("#ReadingRoomID").val() != null){
        $("#ReadingRoomID").attr("data-persisted-value", $("#ReadingRoomID").val());
    }

    // If there are multiple Sites to choose from, disable ReadingRooms until a Site is selected.
    if (!$("#Site").val() && ($("#Site option[value!='']").length > 1)) {
        $("#ReadingRoomID").prop("disabled", true);
        $("#ReadingRoomID").empty();
        return;
    } else {
        $("#ReadingRoomID").prop("disabled", false);
    }

    let postData = {};

    if ($("#Site")) {
        postData.site = $("#Site").val();
    }

    return new Promise((resolve, reject) => {
        $.ajax({
            type: "GET",
            url: "aeon.dll/ajax?query=GetReadingRooms",
            data: postData,
            success: function (response) { 
                if (response.status == "success") {
                    PopulateReadingRooms(response.data);
                    resolve(response.data);
                } else {
                    HandleError(response);
                    reject(response.message);
                }
            },
            error: function () {
                HandleError("Server error. Please refresh the page and try again. If this error persists, please contact staff.");
                reject("Server Error");
            }
        });
    });
}

function PopulateReadingRooms(data) {
    $("#ReadingRoomID").empty();
    window.readingRooms = data;
    readingRooms.forEach( readingRoom => {
        if (readingRoom.OpenHours.length > 0) {
            let option = document.createElement("option");
            option.setAttribute("value", readingRoom.ID);
            option.innerText = readingRoom.ReadingRoomName;
            $("#ReadingRoomID").append(option);
        }
    });
    
    SelectPersistedValue($("#ReadingRoomID"));
    window.readingRoom = readingRooms.filter(obj => { return obj.ID === $("#ReadingRoomID").val() })[0];
    GetAppointments();
}

async function GetAvailability(date, useCache=true, getNextAvailable=false) {
    let dateKey = moment(date).format("YYYY-MM-DD")

    if (readingRoom.appointmentAvailability == null) {
        readingRoom.appointmentAvailability = {};
        readingRoom.availabilityPromises = {};
    }

    if (useCache == false) {
        delete readingRoom.appointmentAvailability[dateKey];
        delete readingRoom.availabilityPromises[dateKey];
    }

    if (dateKey in readingRoom.appointmentAvailability) {
        return readingRoom.appointmentAvailability[dateKey];
    }

    if (!(dateKey in readingRoom.availabilityPromises)){
        readingRoom.availabilityPromises[dateKey] = new Promise((resolve, reject) => {
            $.ajax({
                type: "GET",
                url: "aeon.dll/ajax?query=GetAppointmentAvailability",
                data: {
                    ReadingRoomID: $("#ReadingRoomID").val(), 
                    AppointmentDate: dateKey,
                    GetNextAvailable: getNextAvailable
                },
                success: function (response) {
                    if (response.status == "success") {
                        readingRoom.appointmentAvailability[dateKey] = response.data;
						
						//When editing an appointment, its start and stop time may not appear in availabilities if there are no more seats available.
						//In this case, construct an availability for the current appointment.
						let persistedStartTimeString = $("#StartTime").data("persisted-value");
						let persistedStopTimeString = $("#StopTime").data("persisted-value");
						
						if (persistedStartTimeString && persistedStopTimeString) {
							let persistedStartTime = moment(persistedStartTimeString);
							let persistedStopTime = moment(persistedStopTimeString);
							
							//Only insert appointment availability for the same day
							if (persistedStartTime.format("YYYY-MM-DD") === dateKey) {
								//Determine the maximum appointment length of the persisted option for comparison.
								let persistedAppointmentLength = persistedStopTime.diff(persistedStartTime, 'minutes');
								
								let insertPersistedAvailability = true;
								for (var i = 0; insertPersistedAvailability && (i < readingRoom.appointmentAvailability[dateKey].length); i++) {
									//If we find an availability with a matching starting time, modify the length if necessary and stop looping.
									let availability = readingRoom.appointmentAvailability[dateKey][i];
									
									if (moment(availability["StartTime-ISO8601"]).isSame(persistedStartTime)) {
										insertPersistedAvailability = false;
										availability.MaximumAppointmentLength = Math.max(persistedAppointmentLength, availability.MaximumAppointmentLength);
									}
								}
								
								if (insertPersistedAvailability) {
									let persistedAvailability = {};
									persistedAvailability["StartTime-ISO8601"] = persistedStartTimeString;
									persistedAvailability["MaximumAppointmentLength"] = persistedAppointmentLength;
									persistedAvailability["SeatsAvailable"] = 1;
									readingRoom.appointmentAvailability[dateKey].push(persistedAvailability);
									readingRoom.appointmentAvailability[dateKey].sort( (a, b) => { 
										return moment(a["StartTime-ISO8601"]).diff(moment(b["StartTime-ISO8601"]), "minutes")
									} );
								}
							}
						}
						
						
                        resolve(response.data);
                    } else {
                        HandleError(response);
                        reject(response.message);
                    }
                },
                error: function () {
                    HandleError("Server error. Please refresh the page and try again. If this error persists, please contact staff.");
                    reject("Server Error");
                }
            })
        });
    }

    await readingRoom.availabilityPromises[dateKey];
    return readingRoom.appointmentAvailability[dateKey]
}

function GetAvailabilityForTime(datetime){
    let dateKey = moment(datetime).format("YYYY-MM-DD")
    if (!(dateKey in readingRoom.appointmentAvailability)){ return false; }

    let availability = readingRoom.appointmentAvailability[dateKey].filter(aa => {
        return aa["StartTime-ISO8601"] == moment(datetime).format("YYYY-MM-DDTHH:mm:ss.sss")
    });

    if (availability.length > 0){
        return availability[0];
    }
    return null;
}


async function GetAppointments() {
    let requestLink = null;
    if ($("#RequestLink").length > 0){
        requestLink = $("#RequestLink").val();

        // Disable and hide Appointment select/create if RequestLink is an Activity.
        if (requestLink[0] == 'E') {
            $(".appointments-enabled").addClass("d-none");
            $("#AppointmentID").prop("disabled", true);
            $("#AppointmentID").empty();
            $("#NewAppointment").prop("disabled", true);
            return;
        } else if (requestLink[0] == 'R') {
            $(".appointments-enabled").removeClass("d-none");
            $("#AppointmentID").prop("disabled", false);
            $("#NewAppointment").prop("disabled", false);
        }
    }

    // Disable Appointment select/create if no Reading Room is selected
    if (!$("#ReadingRoomID").val()) {
        $("#AppointmentID").prop("disabled", true);
        $("#NewAppointment").prop("disabled", true);
        $("#AppointmentID").empty();
        return;
    }


    // Create data for the AJAX message
    let postData = { 
        ReadingRoomID: $("#ReadingRoomID").val(),
        Context: defaultAppointmentContext
    };
    if (requestLink != null && requestLink[0] == 'R') {
        postData.ResearcherUsername = requestLink.slice(2);
        postData.Context = "researcher";
    }
    if ($("#TransactionNumber").val()) {
        postData.TransactionNumber = $("#TransactionNumber").val();
    }

    return new Promise((resolve, reject) => {
        $.ajax({
            type: "GET",
            url: "aeon.dll/ajax?query=GetAppointments",
            data: postData,
            success: function (response) {
                if (response.status == "success") {
                    PopulateAppointments(response.data);
                    resolve(response.data);
                } else {
                    HandleError(response);
                    reject(response.message);
                }
            },
            error: function(){ 
                HandleError("Server error. Please refresh the page and try again. If this error persists, please contact staff.");
                reject("Server Error");
            }
        });
    });
}

function PopulateAppointments(appointments) {
	var onEditForm = $("#AppointmentID").length > 0;
	
    if (onEditForm) { $("#AppointmentID").empty(); }
    appointments.forEach( appointment => {
        // Add option to select input
		if (onEditForm) {
			let option = document.createElement("option");
			if (jQuery.isEmptyObject(appointment)) {
				option.setAttribute("value", "");
				option.innerText = "No Appointment";
			} else {
				option.setAttribute("value", appointment.AppointmentID);
				option.innerText = moment(appointment.StartTime).format('MM/DD/YYYY hh:mm a') + " - " + moment(appointment.StopTime).format('hh:mm a');
				var name = appointment.Name;
				if (name) {
					option.innerText = appointment.Name + ': ' + option.innerText;
				}
			}
			$("#AppointmentID").append(option);
		}
        // Add properties used by scheduler
        appointment.text = appointment.Name ?? "Appointment";
        appointment.startDate = appointment.StartTime;
        appointment.endDate = appointment.StopTime;
    });
	if (onEditForm) {
		SelectPersistedValue($("#AppointmentID"));
		$("#NewAppointment").prop("disabled", false);
	}
    readingRoom.appointments = appointments;
}

async function GetValidStartTimes(date) {
    let availability = await GetAvailability(date);
    return availability.map(aa => moment(aa["StartTime-ISO8601"]));
}

function GetValidEndTimes(startTime, availability) {
    if (availability == null) {
        return [];
    }
    
    startTime = moment(startTime);
    let endTime = moment(startTime).add(readingRoom.MinAppointmentLength, 'm');
    let maxEndTime = moment(startTime).add(availability.MaximumAppointmentLength, "m");
    validEndTimes = [];
    while(endTime <= maxEndTime) {
        validEndTimes.push(new moment(endTime));
        endTime = endTime.add(readingRoom.AppointmentIncrement, 'm');
    }
    return validEndTimes;
}


async function BuildSchedulerControl() {

    // Initialize the opening/closing hours
    // We should do this based on the earliest and latest closing hours overall, not just for a particular day,
    // so that the scope of the scheduler doesn't change as the user navigates through dates.
    if (readingRoom.OpenHours.length > 0) {
        //If the reading room has open hours defined, find the earliest/latest time to set the scheduler
        readingRoomEarliestOpeningHour = getMinTime(readingRoom.OpenHours, "OpenTime");
        readingRoomLatestOpeningHour = getMaxTime(readingRoom.OpenHours, "CloseTime");
    }

    let startDate = new moment().add(readingRoom.MinLeadDays, 'd');
    availability = await GetAvailability(startDate, false, true);

    // Find the next day with nonzero availability.
    // This takes place in the DLL now, but is being added here for backward compatibility.
    while (availability.length < 1) {
        startDate.add(1, 'd');
        availability = await GetAvailability(startDate);
    }

    // The first available date may actually fall later than today + MinLeadDays,
    // so we need to reset StartDate accordingly
    startDate = new moment(availability[0].UTCStartTime);

    
    let currentView, currentStartDate;
    if (typeof(window.scheduler) != "undefined" && typeof(window.scheduler.getStartViewDate) == "function") { 
        currentView = scheduler._currentView; 
        currentStartDate = scheduler.getStartViewDate()
    } else {
        currentView = "week";
        currentStartDate = startDate.toDate();
    }


    window.scheduler = $("#scheduler").dxScheduler({
        adaptivityEnabled: true,
        dataSource: readingRoom.appointments,
        views: ["day","week","month"],
        currentView: currentView,
        cellDuration: readingRoom.AppointmentIncrement,
        showCurrentTimeIndicator: true,
        currentDate: currentStartDate,
        shadeUntilCurrentTime: true,
        startDayHour: readingRoomEarliestOpeningHour,
        endDayHour: readingRoomLatestOpeningHour,
        showAllDayPanel: false,
        height: "58vh",
        editing: {
            allowTimeZoneEditing: true,
            allowDeleting: false
        },
        
        onOptionChanged: function(e) {
            if (e.name === 'currentDate') {
                GetAvailability(e.value);
            }

            if (e.name == 'currentView' && e.value == 'day') {
                if (e.component.option("selectedCellData").length > 0) {
                    e.component.option("currentDate", e.component.option("selectedCellData")[0].startDate);
                }
            }
        },

        dataCellTemplate: async function(itemData, itemIndex, itemElement) {
            var date = itemData.startDate;
            var isDisabled = !(await IsValidAppointmentTimeAsync(date));
            var element = $(`<div>${itemData.text}</div>`);

            if (isDisabled) {
                element.addClass('disable-date');
            }

            return itemElement.append(element);
        },

        
        dateCellTemplate: async function(itemData, itemIndex, itemElement) {
            var element = $(`<div>${itemData.text}</div>`);
            var isDisabled = !(await IsValidAppointmentDateAsync(itemData.date));
            if (isDisabled && scheduler._currentView != "month") {
                element.addClass('disable-date');
            };
            return itemElement.append(element);
        },

        timeCellTemplate: function(itemData, itemIndex, itemElement) {
            var element = $(`<div>${itemData.text}</div>`);
            return itemElement.append(element);
        },

        onAppointmentClick: function(e) {
            e.cancel = true;
        },

        onAppointmentFormOpening: function(data) {
            let availability = GetAvailabilityForTime(data.appointmentData.startDate);
            if (availability == null && data.appointmentData.AppointmentID == null) {
                data.cancel = true;
                notifyDisableDate();
            }

            var form = data.form;

            let startDate = moment(data.appointmentData.startDate);
            let endDate = moment(data.appointmentData.endDate);
            
            let dateKey = startDate.format("YYYY-MM-DD")
            let validStartTimes = readingRoom.appointmentAvailability[dateKey].map(aa => moment(aa["StartTime-ISO8601"]));
            let validEndTimes = GetValidEndTimes(startDate, availability);
            
            if (endDate.diff(startDate, "minutes") < readingRoom.MinAppointmentLength) {
                endDate = new moment(startDate).add(readingRoom.MinAppointmentLength, "m");
            }

			//showForProxy hidden when editing existing appointments or creating appointment for a researcher
            let showForProxy = true;

			if (data.appointmentData.AppointmentID == null) {
				let requestLink = $("#RequestLink").val();
				if (requestLink != null && requestLink[0] == 'R') {
					form.updateData("researcherUsername", requestLink.slice(2));
					form.updateData("forProxy", true);
					showForProxy = false;
				}
			} else {
				showForProxy = false;
			}
			
            form.updateData("startDate", startDate);
            form.updateData("endDate", endDate);


            data.popup.option("toolbarItems[0].options.text", "Schedule");

            form.option("items", [
                {
                    label: { text: "Appointment Name" },
                    name: "text",
                    dataField: "text",
                    editorType: "dxTextBox",
                    colSpan: 2
                }, {
                    label: { text: "Date" },
                    name: "appointmentDate",
                    editorType: "dxDateBox",
                    colSpan: showForProxy ? 1 : 2,
                    isRequired: true,
                    editorOptions: {
                        value: startDate.toDate(),
                        onValueChanged: async function(args) {
                            let startTimes = await GetValidStartTimes(args.value);
                            form.getEditor("startDate").option("dataSource", startTimes);
                            form.updateData("startDate", null);
                        },
                        width: "100%",
                        type: "date"
                    }
                }, {
                    label: { text: "Allow proxies to add requests" },
                    name: "forProxy",
                    dataField: "forProxy",
                    editorType: "dxSwitch",
                    visible: showForProxy
                }, {
                    label: { text: "Start Time" },
                    name: "startDate",
                    dataField: "startDate",
                    editorType: "dxSelectBox",
                    isRequired: true,
                    editorOptions: {
                        value: startDate.format("LT"),
                        dataSource: validStartTimes,
                        noDataText: "No appointments available",
                        displayExpr: function(item) { if (item != null) {return item.format("LT")}},
                        onValueChanged: function(args) {
                            form.getEditor("endDate").option("dataSource", GetValidEndTimes(args.value, GetAvailabilityForTime(args.value)));
                            form.updateData("endDate", null);
                        },
                        width: "100%"
                    }
                }, {
                    label: { text: "Stop Time" },
                    name: "endDate",
                    dataField: "endDate",
                    editorType: "dxSelectBox",
                    isRequired: true,
                    editorOptions: {
                        value: endDate.format("LT"),
                        dataSource: validEndTimes,
                        noDataText: "No appointments available",
                        displayExpr: function(item) { if (item != null) {return item.format("LT")}},
                        width: "100%"
                    }
                }
            ]);
        },

        onAppointmentAdding: function(e) {
            let appointment = {
                StartTime: moment(e.appointmentData.startDate).format("YYYY-MM-DDTHH:mm:ss.sss"),
                StopTime: moment(e.appointmentData.endDate).format("YYYY-MM-DDTHH:mm:ss.sss"),
                ReadingRoomID: $("#ReadingRoomID").val(),
                AvailableToProxies: (e.appointmentData.forProxy == true),
                Name: e.appointmentData.text,
                ResearcherUsername: e.appointmentData.researcherUsername
            }

            let [valid, message]= ValidateAppointment(appointment);
            if (!valid) {
                e.cancel = true;
                DevExpress.ui.notify(message, "warning", 5000);
            } else {
                new Promise((resolve, reject) => {
                    $.ajax({
                        type: "POST",
                        url: "aeon.dll/ajax?query=CreateAppointment",
                        data: appointment,
                        success: function (response) {
                            if (response.status == "success") {
                                GetAvailability(appointment.StartTime, false)
                                    .then(RefreshScheduler())
                                    .then(resolve(response.data));
                            } else {
                                DevExpress.ui.notify(response.message, "warning", 5000);
                                RefreshScheduler();
                                reject(response.message);
                            }
                        },
                        error: function () {
                            HandleError("Server error. Please refresh the page and try again. If this error persists, please contact staff.");
                            RefreshScheduler();
                            reject("Server Error");
                        }
                    });
                }).then(newAppointment => {
                    // If we're on a request page,
                    // Requery appointments and select the newly-created one.
                    // Otherwise, set a flag so we know to reload the page when the scheduler is closed.
                    GetAppointments().then( success =>{
                        $apptId = $("#AppointmentID");
                        if ($apptId.length > 0) {
                            $apptId.val(newAppointment.AppointmentID);
                            $apptId.attr("data-persisted-value", newAppointment.AppointmentID);
                            $("#ModalDismiss").click();
                        } else {
                            window.appointmentsChanged = true;
                        }
                    })
                }, reason => {
                    e.cancel = true;
                    DevExpress.ui.notify(reason, "warning", 5000);
                });
            }

        },

        onAppointmentUpdating: function(e) {
            let appointment = {
                AppointmentID: e.newData.AppointmentID,
                StartTime: e.newData.startDate,
                StopTime: e.newData.endDate,
                ReadingRoomID: $("#ReadingRoomID").val(),
                AvailableToProxies: (e.newData.forProxy == true),
                Name: e.newData.text
            }
            let [valid, message]= ValidateAppointment(appointment);
            if (!valid) {
                e.cancel = true;
                DevExpress.ui.notify(message, "warning", 5000);
            } else {
                new Promise((resolve, reject) => {
                    $.ajax({
                        type: "POST",
                        url: "aeon.dll/ajax?query=RescheduleAppointment",
                        data: appointment,
                        success: function (response) {
                            if (response.status == "success") {
                                resolve(response.data);
                            } else {
                                DevExpress.ui.notify(response.message, "warning", 5000);
                                reject(response.message);
                            }
                        },
                        error: function () {
                            HandleError("Server error. Please refresh the page and try again. If this error persists, please contact staff.");
                            reject("Server Error");
                        }
                    });
                }).then(newAppointment => {
                    // On request pages, requery appointments and select the newly-created one.
                    // Otherwise, set a flag so we know to reload the page when the scheduler is closed.
                    if ($("#AppointmentID").length > 0) {
                        GetAppointments().then( success =>{
                            $("#AppointmentID").val(newAppointment.AppointmentID);
                        })
                    } else {
                        window.appointmentsChanged = true;
                    }
                }, reason => {
                    e.cancel = true;
                    DevExpress.ui.notify(reason, "warning", 5000);
                });
            }
        }

    }).dxScheduler("instance");

    function notifyDisableDate() {
        DevExpress.ui.notify("You cannot schedule an appointment at the selected time.", "warning", 5000);
    }

    async function IsValidAppointmentTimeAsync(datetime) {
        if (moment(datetime) < moment()) {
            return false;
        }
        availability = await GetAvailability(datetime);
        return (availability.filter(aa => {
            return aa["StartTime-ISO8601"] == moment(datetime).format("YYYY-MM-DDTHH:mm:ss.sss")
        }).length > 0);
    }

    async function IsValidAppointmentDateAsync(date) {
        availability = await GetAvailability(date);
        return (availability.length > 0);
    }

    function ValidateAppointment(newAppt) {
        let valid = true;
        let message = "";
        
        for (i = 0; i < readingRoom.appointments.length; i++) { 
            existingAppt = readingRoom.appointments[i];
            if (existingAppt.AppointmentID != newAppt.AppointmentID && existingAppt.StartTime < newAppt.StopTime && existingAppt.StopTime > newAppt.StartTime) {
                message += "Unable to schedule an appointment that intersects an existing appointment.\n"
                valid = false;
                break;
            }
        };

        let availability = GetAvailabilityForTime(newAppt.StartTime);
        if (availability == null || availability.length == 0) {
            valid = false;
            message += `${moment(newAppt.StartTime).toLocaleString()} is not an available appointment time.\n`
        } else {
            let maxEndTime = moment(newAppt.StartTime).add(availability.MaximumAppointmentLength, "m");
            if (maxEndTime < moment(newAppt.StopTime)) {
                valid = false;
                message += `An appointment that begins at ${moment(newAppt.StartTime).toLocaleString()} must end by ${maxEndTime.toLocaleString()}\n`;
    
            }
        }
        
        return [valid, message];
    }

}

const getMaxTime = (arr, prop) => {
    const tmp = arr.map(x => moment(x[prop], 'HH:mm:ss').hour());
    return Math.max(...tmp);
 }

const getMinTime = (arr, prop) => {
    const tmp = arr.map(x => moment(x[prop], 'HH:mm:ss').hour());
    return Math.min(...tmp);
}